package rocks.zipcode;

import java.io.*;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author leon on 16/11/2018.
 */
public class Document implements DocumentInterface {

    private final FileWriter fileWriter;
    private final File file;


    public Document(String fileName) throws IOException {
        this(new File(fileName));
    }

    public Document(File file) throws IOException {
        this.file = file;
        this.fileWriter = new FileWriter(file);
    }

    @Override
    public void write(String contentToBeWritten) {

        BufferedWriter bufferedWriter = null;
        try{
            bufferedWriter = new BufferedWriter(new FileWriter(file));
            bufferedWriter.write(contentToBeWritten);
            bufferedWriter.close();
        }catch (IOException e){

        }
    }

    @Override
    public void write(Integer lineNumber, String valueToBeWritten) {
        List<String> lines;
        try {
            lines = Files.readAllLines(file.toPath());
            lines.set(lineNumber, valueToBeWritten);
            Files.write(file.toPath(), lines);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Override
    public String read(Integer lineNumber) {
        String result = "";
        try {
             result =  Files.readAllLines(Paths.get(this.file.getAbsolutePath())).get(lineNumber);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return result;
    }

    @Override
    public String read() {

        BufferedReader bufferedReader = null;
        String line = "";
        String content = "";
        try{
            bufferedReader = new BufferedReader(new FileReader(file));
            while ((line = bufferedReader.readLine())!=null){
                content+=line +"\n";
            }
        }catch (IOException e){
            e.printStackTrace();
        }
        return content.trim();
    }

    @Override
    public void replaceAll(String stringToReplace, String replacementString) {

        String result  = read().replaceAll(stringToReplace,replacementString);
        write(result);
    }

    @Override
    public void overWrite(String content) {
        this.write(content);
    }

    public List<String> toList() {

        Stream<String> fileStream = Stream.of(read());
        return fileStream.collect(Collectors.toList());
    }

    @Override
    public File getFile() {
        return this.file;
    }

    @Override
    public String toString() {
        return getFile() + "{" + read() + "}";
    }

}
